package invoice;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;


/**
 * @author S.Chassande-Barrioz
 */
public class Apps1 extends InvoiceAppsHelper {

    final static String[] colors = {"blue", "red", "yellow", "black", "white"};
    final static char[] sizes = {'L', 'M', 'S'};

    public static void main(String[] args) {
        Apps1 a1 = null;
        try {
            a1 = new Apps1(args[0]);
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        System.out.println("Creating the addresses...");
        a1.createAdresses();
        System.out.println("Creating the products...");
        a1.createProducts();
        System.out.println("Data base:");
        System.out.println("==========");
        a1.printAll();
        System.out.println();
        System.out.println("Creating the invoice...");
        Invoice i = a1.createInvoice();
        System.out.println(i);
        System.out.println();
        System.out.println("Removing the invoice...");
        a1.removeInvoice(i);
        System.out.println("Removing the products...");
        a1.removeProducts();
        System.out.println("Removing the addresses...");
        a1.removeAddresses();
        System.out.println("Data base:");
        System.out.println("==========");
        a1.printAll();
    }

    public Apps1(String propertiesFileName) throws IOException {
        super(propertiesFileName);
    }

    protected String getLoggerName() {
        return "invoice.apps1";
    }


    public void createProducts() {
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        for(int i=0 ; i<colors.length; i++) {
            for(int j=0; j<sizes.length; j++) {
                pm.makePersistent(new Product(
                    "t-shirt_" + colors[i] + "_" + sizes[j],
                    sizes[j], colors[i], 200, 15));
                pm.makePersistent(new Product(
                    "short_" + colors[i] + "_" + sizes[j],
                    sizes[j], colors[i], 100, 10));
                pm.makePersistent(new Product(
                    "sweat-shirt_" + colors[i] + "_" + sizes[j],
                    sizes[j], colors[i], 300, 30));
            }
        }
        pm.currentTransaction().commit();
        pm.close();
    }

    public void createAdresses() {
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.makePersistent(
            new Address("Seb", "edelweiss", "38610", "Gieres", "France"));
        pm.makePersistent(
            new Address("Pascal", "?", "38000", "Saint Ismier", "France"));
        pm.makePersistent(
            new Address("Alex", "?", "38000", "Grenoble", "France"));
        pm.makePersistent(
            new Address("Bruno", "?", "750XX", "Paris", "France"));
        pm.currentTransaction().commit();
        pm.close();
    }

    public Invoice createInvoice() {
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();

        //Fetch the products of the invoice
        Query q = pm.newQuery(Product.class);
        //q.declareParameters("String c");
        q.declareParameters("String c, char s");
        //q.setFilter("(color == c)");
        q.setFilter("((color == c) && (size == s))");
        Collection products = (Collection)
            //q.execute(colors[0]);
            q.execute(colors[0], new Character(sizes[0]));
        ArrayList pus = new ArrayList();
        Iterator it = products.iterator();
        while(it.hasNext()) {
            pus.add(new ProductUnits((Product) it.next(), 20, pus.size()));
        }
        q.closeAll();

        //Fetch the address of the invoice
        q = pm.newQuery(Address.class);
        q.setFilter("(name == n)");
        q.declareParameters("String n");
        Collection addresses = (Collection) q.execute("Seb");
        Address sebAddress = (Address) addresses.iterator().next();
        q.closeAll();

        //Build and make persistent the invoice
        Invoice invoice = new Invoice(217, sebAddress, pus);
        pm.makePersistent(invoice);

        pm.currentTransaction().commit();
        pm.close();

        return invoice;
    }

    private void removeInvoice(Invoice i) {
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.deletePersistentAll(i.getProductUnits());
        pm.deletePersistent(i);
        pm.currentTransaction().commit();
        pm.close();
    }
}
