package invoice;

import org.objectweb.speedo.log.Log;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import javax.jdo.JDOHelper;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import java.util.Properties;
import java.util.Collection;
import java.util.Iterator;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class InvoiceAppsHelper {

    public PersistenceManagerFactory pmf = null;

    public Logger logger;

    public InvoiceAppsHelper(String propertiesFileName) throws IOException {
        Properties p = new Properties();
        p.load(new FileInputStream(propertiesFileName));
        pmf = JDOHelper.getPersistenceManagerFactory(p);
        logger = Log.getLoggerFactory().getLogger(getLoggerName());
        logger.log(BasicLevel.DEBUG,
            "PersistenceManagerFactory instanciated ("
            + pmf.getConnectionURL() + ")");
    }

    protected abstract String getLoggerName();

    public void printAll() {
        PersistenceManager pm = pmf.getPersistenceManager();
        Query q = pm.newQuery(Product.class);
        q.setOrdering("name ascending");
        Collection col = (Collection) q.execute();
        Iterator it = col.iterator();
        int i = 0;
        while(it.hasNext()) {
            Product p = (Product) it.next();
            System.out.println("Product (" + p + ")");
            i++;
        }
        q.closeAll();
        System.out.println("There is " + i + " products.");

        q = pm.newQuery(Address.class);
        col = (Collection) q.execute();
        it = col.iterator();
        i = 0;
        while(it.hasNext()) {
            Address a = (Address) it.next();
            System.out.println("Address (" + a + ")");
            i++;
        }
        q.closeAll();
        System.out.println("There is " + i + " addresses.");

        q = pm.newQuery(Invoice.class);
        col = (Collection) q.execute();
        it = col.iterator();
        i = 0;
        while(it.hasNext()) {
            Invoice in = (Invoice) it.next();
            System.out.println("Invoice (" + in + ")");
            i++;
        }
        q.closeAll();
        System.out.println("There is " + i + " invoices.");


        pm.close();
    }

    public void removeProducts() {
        PersistenceManager pm = pmf.getPersistenceManager();
        //pm.currentTransaction().begin();
        Query q = pm.newQuery(Product.class);

        pm.deletePersistentAll((Collection) q.execute());
        q.closeAll();
        //pm.currentTransaction().commit();
        pm.close();
    }

    public void removeAddresses() {
        PersistenceManager pm = pmf.getPersistenceManager();
        //pm.currentTransaction().begin();
        Query q = pm.newQuery(Address.class);
        pm.deletePersistentAll((Collection) q.execute());
        q.closeAll();
        //pm.currentTransaction().commit();
        pm.close();
    }
}
