/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.api;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PBinder;

/**
 * It defines a Jorm configurator which manages the PClassMapping, the PBinder
 * and the PNamingContext.
 *
 * @author S.Chassande-Barrioz
 */
public interface JormFactory {

    /**
     * This Method is a shortcut to the getPBinder(SpeedoProxy) method. It only
     * does the instanciation of the classname
     * (Class.ForName(classsName).newInstance()) and call the
     * getPBinder(SpeedoProxy) method with the created instance. Then if an
     * instance is availlable it is better to use the other method.
     * @param classname the Jorm class name managed by the wanted PBinder
     * @return the PBinder instance to use for the given jorm class name
     * @throws PException
     */
    PBinder getPBinder(String classname) throws PException;

    /**
     * @return the PBinder instance which manages the identifer of the
     * SpeedoProxy instance
     */
    PBinder getPBinder(SpeedoProxy sp) throws PException;

    /**
     * Retrieves a PClassMapping instance managing a SpeedoProxy. If the
     * PClassMapping does not already exist then it is created, configured and
     * mapped in the PMapper. The PClassMapping configuration contains the
     * PBinder assignement, the PNamingContext assignement for each reference,
     * and the GenClassMapping assignement. The assigned GenClassMapping must
     * also be configured. In other hand the PClassMapping of referenced class
     * is not configured now. Only the PNamingContext is needed.
     * @param sp is the SpeedoProxy instance which the PClassMapping is required
     * @return the PClassMapping which manages a SpeedoProxy
     * @throws PException
     */
    PClassMapping getPClassMapping(SpeedoProxy sp) throws PException;

    /**
     * This Method is a shortcut to the getPClassMapping(SpeedoProxy) method. It only
     * does the instanciation of the classname
     * (Class.ForName(classsName).newInstance()) and call the
     * getPClassMapping(SpeedoProxy) method with the created instance. Then if an
     * instance is availlable it is better to use the other method.
     *
     * Retrieves a PClassMapping instance managing a jorm class. If the
     * PClassMapping does not already exist then it is created, configured and
     * mapped in the PMapper. The PClassMapping configuration contains the
     * PBinder assignement, the PNamingContext assignement for each reference,
     * and the GenClassMapping assignement. The assigned GenClassMapping must
     * also be configured. In other hand the PClassMapping of referenced class
     * is not configured now. Only the PNamingContext is needed.
     * @param classname is the class name of the jorm class which the
     * PClassMapping is required
     * @return the PClassMapping which manages a SpeedoProxy
     * @throws PException
     */
    PClassMapping getPClassMapping(String classname) throws PException;

    /**
     * @return the PNamingContext instance which manages a reference to the
     * SpeedoProxy instance
     */
    PNamingContext getPNamingContext(SpeedoProxy sp) throws PException;

    /**
     * This Method is a shortcut to the getPNamingContext(SpeedoProxy) method.
     * It only does the instanciation of the classname
     * (Class.ForName(classsName).newInstance()) and call the
     * getPNamingContext(SpeedoProxy) method with the created instance. Then if
     * an instance is availlable it is better to use the other method.
     * @param classname the Jorm class name managed by the wanted PNamingContext
     * @return the PNamingContext instance to use for the given jorm class name
     * @throws PException
     */
    PNamingContext getPNamingContext(String classname) throws PException;

    /**
     * It retrieves a PName since a representation of an identifier. The
     * representation can be a PName which the PNamingContext reference must be
     * checked and updated if needed. The identifier representation can be also
     * an encode version of a PName. The cofing format depends on the
     * implementation.
     *
     * @param oid is the identifier representation
     * @return a PName matching the oid representation and valid in a
     * PNameManager
     * @throws PException if the oid does not represent a managed identifier
     * representation.
     */
    PName decode(Object oid) throws PException;

    /**
     * It retrieves a representation of an identifier since a PName.
     *
     * @param pn is the pname to encode
     * @return an identifier representation
     * @throws PException if the pname is malformed (null for example).
     */
    Object encode(PName pn) throws PException;
}
