package org.objectweb.speedo.api;

/**
 * Defines the Speedo options which can be passed as properties to the
 * PersistenceManagerFactory
 */
public interface SpeedoProperties {

    // Constant usable in the Properties of a PersistenceManagerFactory

    /**
     * Is the property value to pass to the PersistenceManagerFactory to
     * indicate that the size is unlimited.
     */
    String NO_LIMIT = "nolimit";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * cache size.
     */
    String CACHE_SIZE = "org.objectweb.speedo.cache";

    /**
     * Is the property to pass to the PersistenceManagerFactory to indicate
     * if the transactionnal environnement is managed.
     */
    String MANAGED = "org.objectweb.speedo.managed";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of PersistenceManager pool.
     */
    String PM_POOL_MIN = "org.objectweb.speedo.peristencemanager.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of PersistenceManager pool.
     */
    String PM_POOL_MAX = "org.objectweb.speedo.peristencemanager.pool.max";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of Query pool.
     */
    String QUERY_POOL_MIN = "org.objectweb.speedo.query.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of Query pool.
     */
    String QUERY_POOL_MAX = "org.objectweb.speedo.query.pool.max";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of compiled query pool.
     */
    String COMPILED_QUERY_POOL_MIN = "org.objectweb.speedo.compiledquery.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of compiled query pool.
     */
    String COMPILED_QUERY_POOL_MAX = "org.objectweb.speedo.compiledquery.pool.max";


    // Constant usable in the .jdo files (keys of the extension tags)

    String VENDOR_NAME = "speedo";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the sql column for a primitive field (not a reference).
     */
    String SQL_NAME = "sql-name";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the foreign keys referencing the current persistent object.
     */
    String SOURCE_FK = "source-foreign-keys";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the foreign keys referencing the referenced persistent object.
     */
    String TARGET_FK = "target-foreign-keys";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the reverse field.
     */
    String REVERSE_FIELD = "reverse-field";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the join table in a multivalued reference (Collection, Map,
     * Array, Set, List).
     */
    String JOIN_TABLE = "join-table";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the class permiting the conversion of a field between the
     * memory and the data support.
     */
    String FIELD_CONVERTER = "field-converter";
}