/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.api;

/**
 * This interface permits to the user to convert a value store a support into
 * the value used in its persistent object. For example you can store a Date
 * value as a long in your data support and view this value as a java.util.Date
 * in youur application. It is the role of the persistent object provider to
 * implement this interface for each field which the type does not match to the
 * type into the data support. This converter can be only used for primitive
 * type, ie the memory type cannot be a reference to a persistent object.
 *
 * @author S.Chassande-Barrioz
 */
public interface UserFieldMapping {

    /**
     * Retrieves the java type corresponding to the type into the data support.
     * @return a Class object (never null).
     */
    Class getStorageType();

    /**
     * Retrieves the java type corresponding to the type in memory.
     * @return a Class object (never null).
     */
    Class getMemoryType();

    /**
     * Converts a value from the data support into a value in memory
     * @param storagevalue is the value store in the support (can be null).
     * @return the value in memory (can be null).
     */
    Object toMemory(Object storagevalue);

    /**
     * Converts a value from the data support into a value in memory
     * @param memoryvalue the value in memory (can be null).
     * @return is the value store in the support (can be null).
     */ 
    Object toStorage(Object memoryvalue);
}
