/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.api;

import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class WorkingSetLifeCycle {

    public final static byte ACTIVE_ACTION = 1;
    public final static byte BEGIN_TX_ACTION = 2;
    public final static byte COMMIT_TX_ACTION = 3;
    public final static byte ROLLBACK_TX_ACTION = 4;
    public final static byte CLOSE_ACTION = 5;


    public static byte getNextStatus(byte status, byte action)
            throws PersistenceException {
        switch (status) {
        case TransactionalPersistenceManager.CTX_ACTIVE:
            switch (action) {
            case ACTIVE_ACTION:
                return TransactionalPersistenceManager.CTX_ACTIVE;
            case BEGIN_TX_ACTION:
                return TransactionalPersistenceManager.CTX_ACTIVE_TRANSACTIONAL;
            case COMMIT_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE,COMMIT_TX_ACTION)");
            case ROLLBACK_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE,ROLLBACK_TX_ACTION)");
            case CLOSE_ACTION:
                return TransactionalPersistenceManager.CTX_CLOSED;
            default:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE, " + action + ")");
            }
        case TransactionalPersistenceManager.CTX_ACTIVE_TRANSACTIONAL:
            switch (action) {
            case ACTIVE_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE_TRANSACTIONAL, ACTIVE_ACTION)");
            case BEGIN_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE_TRANSACTIONAL, BEGIN_TX_ACTION)");
            case COMMIT_TX_ACTION:
                return TransactionalPersistenceManager.CTX_COMMITTED;
            case ROLLBACK_TX_ACTION:
                return TransactionalPersistenceManager.CTX_ABORTED;
            case CLOSE_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE_TRANSACTIONAL, CLOSE_ACTION)");
            default:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ACTIVE_TRANSACTIONAL, " + action + ")");
            }
        case TransactionalPersistenceManager.CTX_COMMITTED:
            switch (action) {
            case ACTIVE_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_COMMITTED, ACTIVE_ACTION)");
            case BEGIN_TX_ACTION:
                return TransactionalPersistenceManager.CTX_ACTIVE_TRANSACTIONAL;
            case COMMIT_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_COMMITTED, COMMIT_TX_ACTION)");
            case ROLLBACK_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_COMMITTED, ROLLBACK_TX_ACTION)");
            case CLOSE_ACTION:
                return TransactionalPersistenceManager.CTX_CLOSED;
            default:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_COMMITTED, " + action + ")");
            }
        case TransactionalPersistenceManager.CTX_ABORTED:
            switch (action) {
            case ACTIVE_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ABORTED, ACTIVE_ACTION)");
            case BEGIN_TX_ACTION:
                return TransactionalPersistenceManager.CTX_ACTIVE_TRANSACTIONAL;
            case COMMIT_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ABORTED, COMMIT_TX_ACTION)");
            case ROLLBACK_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ABORTED, ROLLBACK_TX_ACTION)");
            case CLOSE_ACTION:
                return TransactionalPersistenceManager.CTX_CLOSED;
            default:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_ABORTED, " + action + ")");
            }
        case TransactionalPersistenceManager.CTX_CLOSED:
            switch (action) {
            case ACTIVE_ACTION:
                return TransactionalPersistenceManager.CTX_ACTIVE;
            case BEGIN_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_CLOSED, BEGIN_TX_ACTION)");
            case COMMIT_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_CLOSED, COMMIT_TX_ACTION)");
            case ROLLBACK_TX_ACTION:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_CLOSED, ROLLBACK_TX_ACTION)");
            case CLOSE_ACTION:
                return TransactionalPersistenceManager.CTX_CLOSED;
            default:
                throw new PersistenceException("Illegal action on a transaction"
                        + " (CTX_CLOSED, " + action + ")");
            }
        default:
            throw new PersistenceException("Illegal action on a transaction"
                    + " (unknonw status=" + status + ", action=" + action + ")");
        }
    }

}
