package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.List;

/**
 * Modifies the access of fields
 * @author S.Chassande-Barrioz
 */
public class FieldAccessModifier extends LoggedClassAdapter {

    private int oldAccess;
    private int newAccess;
    private List fieldNames;

    public FieldAccessModifier(ClassVisitor classVisitor,
                               Logger logger,
                               int oldaccess,
                               int newaccess,
                               List fieldnames) {
        super(classVisitor, logger);
        oldAccess = oldaccess;
        newAccess = newaccess;
        fieldNames = fieldnames;
        logger.log(BasicLevel.DEBUG, " old:" + oldAccess
            + " / new: " + newAccess
            + " / fieldNames: " + fieldNames);
    }

    /**
     * @param access
     * @param name
     * @param desc
     * @param value
     */
    public void visitField(final int access,
                           final String name,
                           final String desc,
                           final Object value) {
        int ac = (fieldNames.contains(name) && ((access & oldAccess) != 0)
            ?  access - oldAccess + newAccess
            : access);
        logger.log(BasicLevel.DEBUG, "Field " + name + " old:" + access + " / new: " + ac);
        super.visitField(ac, name, desc, value);
    }
}
