package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.util.monolog.api.Logger;

import java.util.List;

public class MergedClassModifier extends LoggedClassAdapter {

    private List itfs;

    private String persistenceCapableSuperClass;

    private boolean fixProxyMethodOverriding;

    public MergedClassModifier(ClassVisitor classVisitor,
                               List itfs,
                               Logger logger) {
        super(classVisitor, logger);
        this.itfs = itfs;
        this.fixProxyMethodOverriding = false;
    }

    public MergedClassModifier(ClassVisitor classVisitor,
                               List itfs,
                               String persistenceCapableSuperClass,
                               Logger logger) {
        super(classVisitor, logger);
        this.itfs = itfs;
        this.fixProxyMethodOverriding = true;
        this.persistenceCapableSuperClass = persistenceCapableSuperClass;
    }

    // IMPLEMENTATION OF THE ClassVisitor INTERFACE //
    // ---------------------------------------------//

    public void visit(final int access,
                      final String name,
                      final String superName,
                      final String[] interfaces,
                      final String sourceFile) {
        // just stores the interfaces implemented by the visited class
        if (interfaces != null) {
            for (int i = 0; i < interfaces.length; ++i) {
                if (!itfs.contains(interfaces[i])) {
                    itfs.add(interfaces[i]);
                }
            }
        }
    }

    public CodeVisitor visitMethod(final int access,
                                   final String name,
                                   final String desc,
                                   final String[] exceptions) {
        if (name.equals("<init>") || (access & Constants.ACC_ABSTRACT) != 0) {
            // do not merge constructors and abstract methods
            return null;
        } else {
            CodeVisitor c = cv.visitMethod(access, name, desc, exceptions);
            if (fixProxyMethodOverriding) {
                if (name.equals("jdoLoadFieldsInAccessor")
                        || name.equals("jdoLoadFieldsFromAccessor")
                        || name.equals("jdoMakePersistent")) {
                    c.visitVarInsn(Constants.ALOAD, 0);
                    c.visitVarInsn(Constants.ALOAD, 1);
                    c.visitMethodInsn(
                            Constants.INVOKESPECIAL,
                            persistenceCapableSuperClass,
                            name,
                            desc);
                } else if (name.equals("jdoLoadFields")) {
                    c.visitVarInsn(Constants.ALOAD, 0);
                    c.visitVarInsn(Constants.ALOAD, 1);
                    c.visitVarInsn(Constants.LLOAD, 2);
                    c.visitMethodInsn(
                            Constants.INVOKESPECIAL,
                            persistenceCapableSuperClass,
                            name,
                            desc);
                } else if (name.equals("jdoMakeFieldPersistent")) {
                    c.visitVarInsn(Constants.ALOAD, 0);
                    c.visitVarInsn(Constants.ILOAD, 1);
                    c.visitVarInsn(Constants.ALOAD, 2);
                    c.visitMethodInsn(
                            Constants.INVOKESPECIAL,
                            persistenceCapableSuperClass,
                            name,
                            desc);
                }
            }
            return c;
        }
    }

    public void visitEnd() {
        // does nothing
    }
}
