package org.objectweb.speedo.generation.enhancer;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoEnhancerException;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.asm.ClassReader;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

public class MetaDataAnalyzer extends EnhancerComponent {

    public final static String LOGGER_NAME
            = "org.objectweb.speedo.generation.enhancer.analyzer";

    /**
     * Initializes this AccessEnhancer
     */
    public void init() {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        //TODO: manage classes localized in a jar file
        isSrcJar = false;
    }

    /**
     * Loads all binary classes described by the Object Model and applies
     * revelant modification to each of them.
     *
     * @exception org.objectweb.speedo.api.SpeedoEnhancerException if something goes wrong
     */
    public void process() throws SpeedoException {

        if (scp.xmldescriptor == null || scp.xmldescriptor.isEmpty())
            throw new SpeedoEnhancerException("xmldescriptor not initialised");

        ArrayList except = new ArrayList();
        Collection xmls = scp.xmldescriptor.values();

        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itPack = desc.jdoPackage.values().iterator(); itPack.hasNext();) {
                SpeedoPackage sp = (SpeedoPackage) itPack.next();
                for (Iterator itclass = sp.jdoClass.values().iterator(); itclass.hasNext();) {
                    SpeedoClass jdoclass = (SpeedoClass) itclass.next();

                    ClassReader cr = null;
                    if (scp.keepclasses) {
                        cr = loadJavaClass(
                                isSrcJar,
                                jdoclass.jdoPackage.name + '.' + jdoclass.name,
                                scp.input,
                                false,
                                scp.output);

                    } else {
                        cr = loadJavaClass(
                                isSrcJar,
                                jdoclass.jdoPackage.name + '.' + jdoclass.name,
                                scp.input,
                                false);
                    }
                    cr.accept(new ClassInfoAnalyzer(this, jdoclass, xmls, logger), true);
                }
            }
        }

        // Displays thrown exceptions
        if (!except.isEmpty() && logger.isLoggable(BasicLevel.WARN)) {
            for (Iterator it = except.iterator(); it.hasNext();) {
                Exception e = (Exception) it.next();
                logger.log(BasicLevel.WARN, e.getLocalizedMessage());
            }
        }
    }
}
