package org.objectweb.speedo.generation.enhancer;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoEnhancerException;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.Constants;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

/**
 * This class transforms the access of fields
 * @author S.Chassande-Barrioz
 */
public class PrivateFieldModifier extends EnhancerComponent {

    public final static String LOGGER_NAME
        = "org.objectweb.speedo.generation.enhancer.privateFieldMdodifier";

    protected int oldAccess;
    protected int newAccess;

    public PrivateFieldModifier() {
        this(Constants.ACC_PRIVATE, Constants.ACC_PROTECTED);
    }

    public PrivateFieldModifier(int oldAccess, int newAccess) {
        this.oldAccess = oldAccess;
        this.newAccess = newAccess;
    }

    public void init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
    }

    public void process() throws SpeedoException {
        if (scp.xmldescriptor == null || scp.xmldescriptor.isEmpty())
            throw new SpeedoEnhancerException("xmldescriptor not initialised");
        Collection xmls = scp.xmldescriptor.values();
        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itclass = desc.getSpeedoClasses().iterator(); itclass.hasNext();) {
                SpeedoClass jdoclass = (SpeedoClass) itclass.next();
                List privfields = new ArrayList();
                Iterator it = jdoclass.jdoField.values().iterator();
                while (it.hasNext()) {
                    SpeedoField sf = (SpeedoField) it.next();
                    if ((sf.access & Constants.ACC_PRIVATE) != 0) {
                        logger.log(BasicLevel.DEBUG,
                            jdoclass.getFQName() + "/" + sf.name + " is private");
                        privfields.add(sf.name);
                    }
                }
                if (privfields.size() == 0) {
                    continue;
                }
                ClassWriter cw = new ClassWriter(false);
                FieldAccessModifier fam = new FieldAccessModifier(
                    cw, logger, oldAccess, newAccess, privfields);
                String name = jdoclass.getFQName();
                ClassReader cr = loadJavaClass(false, name, scp.output, false);
                cr.accept(fam, false);
                writeJavaClass(name, cw, scp.output);
            }
        }
    }
}
