/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */

package org.objectweb.speedo.generation.generator.fields;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.comp.generator.lib.CommonHelper;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoGenerationException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.api.UserFieldMapping;
import org.objectweb.speedo.generation.generator.lib.AbstractVelocityGenerator;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.speedo.metadata.SpeedoArray;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoCollection;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoMap;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.util.monolog.wrapper.velocity.VelocityLogger;

import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class is used to generate accessor classes. This new file will
 * manage status of persistent classes and values of persistent fields.
 *
 * @author	F. Dhumeaux, S.Chassande-Barrioz
 */
public class FieldsGenerator extends AbstractVelocityGenerator {

    public final static String LOGGER_NAME
            = "org.objectweb.speedo.generation.generator.fields";

    public final static String TEMPLATE_NAME = TEMPLATE_DIR + ".fields.Fields";


    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//

    public void init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
    }

    // IMPLEMENTATION OF THE VelocityGenerator INTERFACE //
    //---------------------------------------------------//

    /**
     * This method generates the a file since a SpeedoClass meta object.
     * @param sClass is the speedo meta object.
     * @param fileName name of the new file.
     * @exception SpeedoGenerationException If there is a problem during writing
     * the new file.
     */
    public void generate(SpeedoClass sClass, String fileName)
            throws SpeedoException {
        //EnhancerTool.logger = logger;
        if (templateFN == null)
            templateFN = TEMPLATE_NAME.replace('.', fs) + ".vm";
        try {
            if (!ve.templateExists(templateFN)) {
                templateFN = templateFN.replace(fs, '/');
                if (!ve.templateExists(templateFN)) {
                    throw new SpeedoException("The Speedo template " + templateFN
                        + " is not availlable in the classpath");
                }
            }
            Context ctx = getContext(sClass);
            FileWriter fw = new FileWriter(fileName);
            ve.getTemplate(templateFN).merge(ctx, fw);
            ve.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, new VelocityLogger(logger));
            fw.flush();
            fw.close();
        } catch (Exception e) {
            throw new SpeedoGenerationException(
                    "Error during the generation of the file " + fileName, e);
        }
    }


    // HELPER METHODS FOR THE GENERATION //
    //-----------------------------------//

    public boolean isNull(Object o) {
        return o == null;
    }


    // PRIVATE METHODS //
    //-----------------//

    /**
     * This method initialises the Velocity context.
     *
     * @param jdoClass the jdoClass which represents the class to generate.
     * @return a Velocity context.
     */
    private Context getContext(SpeedoClass jdoClass) throws SpeedoException {

        //creation of the Velocity context
        Context ctx = new VelocityContext();
        ctx.put("tools", this);
        ctx.put("jdoClass", jdoClass);

        ctx.put("package", jdoClass.jdoPackage.name);
        ctx.put("classNameFields", NamingRules.fieldsName(jdoClass.name));
        ctx.put("className", NamingRules.proxyName(jdoClass.name));
        ctx.put("classNameAccessor", NamingRules.accessorName(jdoClass.name));
        ctx.put("fieldList", jdoClass.jdoField);
        String sc = jdoClass.persistenceCapableSuperClassName;
        if (sc != null && sc.length() > 0) {
            ctx.put("superclass", NamingRules.fieldsName(sc));
        } else {
            ctx.put("superclass", "BasicSpeedoAccessor");
        }

        //methods to be implemented from interface Accessor
        ArrayList references = new ArrayList();
        ArrayList primitives = new ArrayList();
        CommonHelper jormTools = new CommonHelper();
        for (Iterator jormit = jdoClass.jormclass.iterateField(); jormit.hasNext();) {
            TypedElement te = (TypedElement) jormit.next();
            SpeedoField field = (SpeedoField) jdoClass.jdoField.get(te.getName());
            Field f = new Field();
            if (field == null) {
                if (te instanceof ScalarField) {
                    //The identifier of the container are not visible in the
                    // jdo instance
                    continue;
                } else {
                    StringBuffer fns = new StringBuffer("{");
                    for (Iterator it = jdoClass.jdoField.values().iterator(); it.hasNext();) {
                        SpeedoField sp = (SpeedoField) it.next();
                        fns.append(sp.type());
                        fns.append(" ");
                        fns.append(sp.name);
                        fns.append(", ");
                    }
                    fns.replace(fns.length() - 2, fns.length(), "}");
                    throw new SpeedoException("The field named " + te.getName()
                            + " is declared as a persistent field in the jorm"
                            + "descriptor (.pd) but it is not availlable in the"
                            + " class " + jdoClass.jormclass.getFQName()
                            + ". Existing fields: " + fns);
                }
            }
            f.declaration = field.publicSignature();
            f.name = field.name;
            f.type = field.type();
            f.isReference = te instanceof Reference;
            f.jormmeth = jormTools.upperFL(te.getName());
            f.jormfield = te.getName();
            if (f.isReference) {
                references.add(f);
                f.isMultiValued = te instanceof GenClassRef;
                f.jormtype = "Object";
                if (f.isMultiValued) {
                    if (field.jdoTuple instanceof SpeedoCollection)
                        f.jormcast = "java.util.Collection";
                    else if (field.jdoTuple instanceof SpeedoMap)
                        f.jormcast = "java.util.Map";
                    else if (field.jdoTuple instanceof SpeedoArray)
                        f.jormcast = te.getType().getJormName()+"[]";
                    else
                        f.jormcast = "Object";
                } else {
                    f.jormcast = te.getType().getJormName();
                }
            } else {
                primitives.add(f);
                f.jormtype = te.getType().getJavaName();
                f.jormcast = field.type();

                f.toMemory = "val";
                f.toStorage = f.name;
                SpeedoExtension se = field.getExtensionByKey(
                        SpeedoProperties.FIELD_CONVERTER);
                if (se != null) {
                    f.userFieldMapping = se.value;
                    try {
                        UserFieldMapping ufm = (UserFieldMapping)
                                java.lang.Class.forName(se.value).newInstance();
                        if (ufm.getStorageType().isPrimitive()) {
                            f.toMemory = "new "
                                    + getJavaLangType(ufm.getStorageType())
                                    + "(" + f.toMemory + ")";
                        }
                        if (ufm.getMemoryType().isPrimitive()) {
                            f.toStorage = "new "
                                    + getJavaLangType(ufm.getMemoryType())
                                    + "(" + f.toStorage + ")";
                        }
                        f.toMemory = f.name + "UFM.toMemory(" + f.toMemory + ")";
                        f.toStorage = f.name + "UFM.toStorage(" + f.toStorage + ")";
                        if (ufm.getStorageType().isPrimitive()) {
                            f.toStorage = "(("
                                    + getJavaLangType(ufm.getStorageType())
                                    + ") " + f.toStorage + ")."
                                    + getValueGetter(ufm.getStorageType()) + "()";
                        } else {
                            f.toStorage = "("
                                    + ufm.getStorageType().getName()
                                    + ") " + f.toStorage;
                        }
                        if (ufm.getMemoryType().isPrimitive()) {
                            f.toMemory = "(("
                                    + getJavaLangType(ufm.getMemoryType())
                                    + ") " + f.toMemory + ")."
                                    + getValueGetter(ufm.getMemoryType()) + "()";
                        } else {
                            f.toMemory = "("
                                    + ufm.getMemoryType().getName()
                                    + ") " + f.toMemory;
                        }
                    } catch (Exception e) {
                        throw new SpeedoException(
                                "Impossible to instanciate the UserFieldMapping class '"
                                + se.value + "' for the field '" + field.name
                                + "' of the class '" + jdoClass.getFQName() + "':", e);
                    }
                }
            }
        }
        ctx.put("references", references);
        ctx.put("primitives", primitives);
        return ctx;
    }

    public boolean isParentField(SpeedoClass sc, String fn) {
        if (sc.persistenceCapableSuperClassName == null
                || sc.persistenceCapableSuperClassName.length() == 0)
            return false;
        SpeedoClass psc = sc.jdoPackage.jdoXMLDescriptor.getSpeedoClass(
                sc.persistenceCapableSuperClassName);
        return psc != null && psc.jdoField.get(fn) != null;
    }

    private String getValueGetter(java.lang.Class c) {
        return c.getName() + "Value";
    }

    public final java.lang.Class[][] PRIMITIVE_TYPES = {
        {Boolean.TYPE, Boolean.class},
        {Byte.TYPE, Byte.class},
        {Character.TYPE, Character.class},
        {Short.TYPE, Short.class},
        {Integer.TYPE, Integer.class},
        {Long.TYPE, Long.class},
        {Float.TYPE, Float.class},
        {Double.TYPE, Double.class}
    };

    private String getJavaLangType(java.lang.Class c) {
        for(int i=0; i<PRIMITIVE_TYPES.length; i++) {
            if (c == PRIMITIVE_TYPES[i][0]) {
                return PRIMITIVE_TYPES[i][1].getName();
            }
        }
        return null;
    }

    public class Field {
        public String jormfield;
        public String jormtype;
        public String jormcast;
        public String jormmeth;
        public String declaration;
        public String name;
        public String type;
        public boolean isMultiValued = false;;
        public boolean isReference = false;;
        public String toMemory = null;
        public String toStorage = null;
        public String userFieldMapping = null;

        public String getJormfield() {
            return jormfield;
        }

        public String getJormtype() {
            return jormtype;
        }

        public String getJormcast() {
            return jormcast;
        }

        public String getJormmeth() {
            return jormmeth;
        }

        public String getDeclaration() {
            return declaration;
        }

        public String getName() {
            return name;
        }

        public String getType() {
            return type;
        }

        public boolean getIsMultiValued() {
            return isMultiValued;
        }

        public boolean getIsReference() {
            return isReference;
        }

        public String getToMemory() {
            return toMemory;
        }

        public String getToStorage() {
            return toStorage;
        }

        public String getUserFieldMapping() {
            return userFieldMapping;
        }
    }

}
