/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 0.1
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: P. Dechamboux, Y. Devaux, F. Dhumeaux.
 *
 */
package org.objectweb.speedo.runtime.concurrence.optimistic;

import org.objectweb.speedo.api.LifeCycle;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.api.Transaction;
import org.objectweb.perseus.cache.api.UnFixProtocolException;

/**
 * This class is the basic implementation of the SpeedoAccessor interface. It
 * is used at the top of the XXXFields class inheritance.
 *
 * @author S.Chassande-Barrioz
 */
public abstract class BasicSpeedoAccessor implements SpeedoAccessor {

    protected byte jdoStatus = LifeCycle.initState();

    protected SpeedoProxy proxy = null;

    protected Transaction tx = null;

    // IMPLEMENTATION OF THE CacheEntry INTERFACE //
    //-------------------------------------------//

    public Object getCeObject() {
        return this;
    }

    public Object getCeIdentifier() {
        return proxy.getCeIdentifier();
    }


    // IMPLEMENTATION OF THE FixableCacheEntry INTERFACE //
    //---------------------------------------------------//

    public void fixCe() {
        proxy.fixCe();
    }

    public void unfixCe() throws UnFixProtocolException {
        proxy.unfixCe();
    }

    public int getCeFixCount() {
        return proxy.getCeFixCount();
    }


    // IMPLEMENTATION OF THE ReplaceableCacheEntry INTERFACE //
    //-------------------------------------------------------//

    public long getCeAge() {
        return proxy.getCeAge();
    }

    public void setCeAge(long l) {
        proxy.setCeAge(l);
    }


    // IMPLEMENTATION OF THE PAccessor INTERFACE //
    //-------------------------------------------//

    public Object getMemoryInstance() {
        return this;
    }


    // IMPLEMENTATION OF THE SpeedoAccessor INTERFACE //
    //------------------------------------------------//

    /**
     * Changes the status of this object
     *
     * @see LifeCycle
     * @param action the action that may change the status
     */
    public void jdoChangeStatus(byte action) {
        jdoStatus = LifeCycle.nextStatePersistenceCapable(jdoStatus, action);
    }

    /**
     * Gets the current status of this object
     *
     * @see LifeCycle
     * @return the current state in the life cycle
     */
    public byte jdoGetStatus() {
        return jdoStatus;
    }

    /**
     * Forces the new value of the status
     *
     * @see LifeCycle
     * @param newValue the new status of this object
     */
    public void jdoSetStatus(byte newValue) {
        jdoStatus = newValue;
    }

    /**
     * @return The SpeedoProxy attached to this state representation.
     */
    public SpeedoProxy jdoGetSpeedoProxy() {
        return proxy;
    }

    /**
     * It assignes the SpeedoProxy attached to this state representation.
     */
    public void jdoSetSpeedoProxy(SpeedoProxy sp) {
        proxy = sp;
    }

    public Transaction jdoGetTransaction() {
        return tx;
    }

    public void jdoSetTransaction(Transaction tx) {
        this.tx = tx;
    }
}
