/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.mapper;

import org.objectweb.jorm.runtime.MapperJCA;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.lib.JormManager;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.comp.compiler.lib.JormCompiler;
import org.objectweb.jorm.comp.compiler.lib.JormCompilerConfigurator;
import org.objectweb.jorm.comp.compiler.lib.JormCompilerParameter;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.medor.eval.prefetch.lib.PrefetchCacheImpl;

import javax.jdo.JDOUserException;

public class JCAMapper extends MapperJCA
        implements MapperAttributes, LifeCycleController {

    public final static String LOGGER_FACTORY_BINDING = "logger-factory";
    public final static String LOGGER_NAME
            = "org.objectweb.speedo.rt.mapper.jca";
    public final static String RESOUCE_ADAPTER_BINDINNG = "resouce-adapter";

    private boolean started;

    private boolean initialized = false;

    // IMPLEMENTATION OF THE UserBindingController INTERFACE //
    //-------------------------------------------------------//

    public Object getFcBindings(String s) {
        if (LOGGER_FACTORY_BINDING.equals(s))
            return getLoggerFactory();
        else if (RESOUCE_ADAPTER_BINDINNG.equals(s))
            return getConnectionFactory();
        return null;
    }

    public void addFcBinding(String s, Object o) {
        if (LOGGER_FACTORY_BINDING.equals(s))
            setLoggerFactory((LoggerFactory) o);
        else if (RESOUCE_ADAPTER_BINDINNG.equals(s))
            try {
                setConnectionFactory(o);
            } catch (PException e) {
                throw new JDOUserException(
                        "Impossible to assign the connection factory to the mapper", e);
            }
    }

    public void removeFcBinding(String s, Object o) {
        if (LOGGER_FACTORY_BINDING.equals(s))
            setLoggerFactory(null);
        else if (RESOUCE_ADAPTER_BINDINNG.equals(s))
            try {
                setConnectionFactory(null);
            } catch (PException e) {
                throw new JDOUserException(
                        "Impossible to remove the connection factory to the mapper", e);
            }
    }


    // IMPLEMENTATION OF THE LifeCycleController INTERFACE //
    //-----------------------------------------------------//

    public String getFcState() {
        return started ? STARTED : STOPPED;
    }

    public void startFc() {
        started = true;
        if (!initialized) {
            try {
                JormCompiler jormcompiler = new JormCompiler();
                JormCompilerParameter jcp = jormcompiler.getCompilerParameter();
                JormCompilerConfigurator jcc = jormcompiler.getCompilerConfigurator();
                jcc.configure();
                jcp.loadConfFile(jcc.getGlobalJormcOptsFile(), jcc.knownMappers());
                jcc.setLoggerFactory(getLoggerFactory());
                jormcompiler.setupLogger();
                jormcompiler.setupMIManager();
                Manager mgr = jormcompiler.getMIManager();
                setMetaInfoManager(mgr);
                setPTypeSpace(mgr.getPTypeSpace());
                setPrefetchCache(new PrefetchCacheImpl());
                initialized = true;
            } catch (PException e) {
                throw new RuntimeException(
                        "Impossible to configure the mapper: "
                        + ExceptionHelper.getNested(e).getMessage());
            }
        }
    }

    public void stopFc() {
        started = false;
    }

}
