/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.storage;

import org.objectweb.perseus.persistence.api.StorageManager;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.fractal.api.control.UserBindingController;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

public class SpeedoStorageManager
        implements StorageManager, UserBindingController {

    public final static String LOGGER_FACTORY_BINDING = "logger-factory";
    public final static String LOGGER_NAME = "org.objectweb.speedo.rt.dom";

    protected LoggerFactory loggerFactory = null;

    protected Logger logger = null;

    //IMPLEMENTATION OF THE UserBindingControler INTERFACE //
    //------------------------------------------------------//

    public Object getFcBindings(String s) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            return loggerFactory;
        }
        return null;
    }

    public void addFcBinding(String s, Object o) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            loggerFactory = (LoggerFactory) o;
            if (loggerFactory != null)
                logger = loggerFactory.getLogger(LOGGER_NAME);
        }
    }

    public void removeFcBinding(String s, Object o) {
        if (LOGGER_FACTORY_BINDING.equals(s)) {
            loggerFactory = null;
        }
    }


    public Object export(Object context, Object obj) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "export ctx=" + context + " / obj=" + obj);
        SpeedoAccessor sa = (SpeedoAccessor) obj;
        SpeedoProxy sp = sa.jdoGetSpeedoProxy();
        try {
            if (sp.getStatus() == PBinding.LIFECYCLE_DELTOWRITE) {
                sp.write(context, sa);
            }
            return sp.export(context);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public Object export(Object context, Object obj, Object hints) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "export ctx=" + context
                    + " / obj=" + obj + " / hints=" + hints);
        SpeedoAccessor sa = (SpeedoAccessor) obj;
        SpeedoProxy sp = sa.jdoGetSpeedoProxy();
        try {
            if (sp.getStatus() == PBinding.LIFECYCLE_DELTOWRITE) {
                sp.write(context, sa);
            }
            return sp.export(context, hints);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public void unexport(Object context, Object oid) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG,
                    "unexport ctx=" + context + " / oid=" + oid);
        try {
            ((PName) oid).unexport(context);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public void unexport(Object context, Object oid, Object hints) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG,
                    "unexport ctx=" + context + " / oid=" + oid);
        try {
            ((PName) oid).unexport(context, hints);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public void read(Object context, Object oid, Object obj) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG,
                    "read ctx=" + context + " / oid=" + oid + " / obj=" + obj);
        SpeedoAccessor sa = (SpeedoAccessor) obj;
        SpeedoProxy sp = sa.jdoGetSpeedoProxy();
        try {
            sp.read(context, sa);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public void read(Object conn, Object oid, Object obj, Object tx) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG,
                    "read conn=" + conn + " / oid=" + oid + " / obj=" + obj + " / tx=" + tx);
        SpeedoAccessor sa = (SpeedoAccessor) obj;
        SpeedoProxy sp = sa.jdoGetSpeedoProxy();
        try {
            sp.read(conn, sa, tx);
        } catch (PException e) {
            throw new PersistenceException(e);
        }
    }

    public void write(Object context, Object oid, Object obj) throws PersistenceException {
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG,
                    "write ctx=" + context + " / oid=" + oid + " / obj=" + obj);
        }
        SpeedoAccessor sa = (SpeedoAccessor) obj;
        SpeedoProxy sp = sa.jdoGetSpeedoProxy();
        logger.log(BasicLevel.DEBUG, "sp.pname=" + sp.getPName());
        switch (sp.getStatus()) {
        case PBinding.LIFECYCLE_ACTIVEFORIO:
        case PBinding.LIFECYCLE_DELTOWRITE:
        case PBinding.LIFECYCLE_NEWTOWRITE:
            try {
                sp.write(context, sa);
            } catch (PException e) {
                throw new PersistenceException(e);
            }
            break;
        case PBinding.LIFECYCLE_NOTBOUND:
        case PBinding.LIFECYCLE_ERROR:
        }
    }
}
